@@ -207,20 +207,10 @@ public class ClassReader {
   // Public methods
   // --------------------------------------------------------------------------
 
-  /**
-   * Returns the major and minor version numbers of this class.
-   *
-   * @return the <tt>int</tt> array { major version, minor version }  
-   */
-
-  public int[] getVersion () {
-    return new int[] { readShort(6), readShort(4) };
-  }
-
   /**
    * Makes the given visitor visit the Java class of this {@link ClassReader
    * ClassReader}. This class is the one specified in the constructor (see
-   * {@link #ClassReader ClassReader}).
+   * {@link #ClassReader(byte[]) ClassReader}).
    *
    * @param classVisitor the visitor that must visit this class.
    * @param skipDebug <tt>true</tt> if the debug information of the class must
@@ -239,7 +229,7 @@ public class ClassReader {
   /**
    * Makes the given visitor visit the Java class of this {@link ClassReader
    * ClassReader}. This class is the one specified in the constructor (see
-   * {@link #ClassReader ClassReader}).
+   * {@link #ClassReader(byte[]) ClassReader}).
    *
    * @param classVisitor the visitor that must visit this class.
    * @param attrs prototypes of the attributes that must be parsed during the
@@ -264,6 +254,7 @@ public class ClassReader {
 
     // visits the header
     u = header;
+    int version = readInt(4);
     int access = readUnsignedShort(u);
     String className = readClass(u + 2, c);
     v = items[readUnsignedShort(u + 4)];
@@ -308,7 +299,7 @@ public class ClassReader {
         w = v + 6;
       } else {
         attr = readAttribute(
-          attrs, attrName, v + 6, readInt(v + 2), c, -1, null);
+            attrs, attrName, v + 6, readInt(v + 2), c, -1, null);
         if (attr != null) {
           attr.next = clattrs;
           clattrs = attr;
@@ -318,7 +309,7 @@ public class ClassReader {
     }
     // calls the visit method
     classVisitor.visit(
-      access, className, superClassName, implementedItfs, sourceFile);
+      version, access, className, superClassName, implementedItfs, sourceFile);
 
     // visits the inner classes info
     if (w != 0) {
@@ -766,9 +757,19 @@ public class ClassReader {
       }
     }
     // visits the class attributes
-    while (clattrs != null) {
-      classVisitor.visitAttribute(clattrs);
-      clattrs = clattrs.next;
+    Attribute last = null;
+    attr = clattrs;
+    while (attr != null) {
+      Attribute next = attr.next;
+      attr.next = last;
+      last = attr;
+      attr = next;
+    }    
+    while (last != null) {
+      attr = last.next;
+      last.next = null;
+      classVisitor.visitAttribute(last);
+      last = attr;
     }
     // visits the end of the class
     classVisitor.visitEnd();
@@ -982,12 +983,12 @@ public class ClassReader {
   }
 
   /**
-   * Reads an attribute in {@link #b b}. The default implementation of this
-   * method returns <tt>null</tt> for all attributes.
+   * Reads an attribute in {@link #b b}.
    *
    * @param attrs prototypes of the attributes that must be parsed during the
    *      visit of the class. Any attribute whose type is not equal to the type
-   *      of one the prototypes will be ignored.
+   *      of one the prototypes is ignored (i.e. an empty {@link Attribute}
+   *      instance is returned).
    * @param type the type of the attribute.
    * @param off index of the first byte of the attribute's content in {@link #b
    *      b}. The 6 attribute header bytes, containing the type and the length
@@ -995,7 +996,7 @@ public class ClassReader {
    *      been read).
    * @param len the length of the attribute's content.
    * @param buf buffer to be used to call {@link #readUTF8 readUTF8}, {@link
-   *      #readClass readClass} or {@link #readConst readConst}.
+   *      #readClass(int,char[]) readClass} or {@link #readConst readConst}.
    * @param codeOff index of the first byte of code's attribute content in
    *      {@link #b b}, or -1 if the attribute to be read is not a code
    *      attribute. The 6 attribute header bytes, containing the type and the
@@ -1020,6 +1021,6 @@ public class ClassReader {
         return attrs[i].read(this, off, len, buf, codeOff, labels);
       }
     }
-    return null;
+    return new Attribute(type);
   }
 }
